@@ -1,9 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import pytest
 import astropy.units as u
-from ....utils.testing import assert_quantity_allclose, requires_data
-from ....maps import WcsGeom
-from .. import JFactory, profiles, DMAnnihilation
+from gammapy.astro.darkmatter import (
+    DarkMatterAnnihilationSpectralModel,
+    JFactory,
+    profiles,
+)
+from gammapy.maps import WcsGeom
+from gammapy.utils.testing import assert_quantity_allclose, requires_data
 
 
 @pytest.fixture(scope="session")
@@ -17,15 +21,14 @@ def jfact(geom):
     return jfactory.compute_jfactor()
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_dmfluxmap(jfact):
-
     emin = 0.1 * u.TeV
     emax = 10 * u.TeV
     massDM = 1 * u.TeV
     channel = "W"
 
-    diff_flux = DMAnnihilation(mass=massDM, channel=channel)
+    diff_flux = DarkMatterAnnihilationSpectralModel(mass=massDM, channel=channel)
     int_flux = (jfact * diff_flux.integral(emin=emin, emax=emax)).to("cm-2 s-1")
     actual = int_flux[5, 5]
     desired = 1.94839226e-12 / u.cm ** 2 / u.s
