@@ -1,18 +1,23 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """HESS Galactic plane survey (HGPS) catalog."""
-from collections import OrderedDict
 import numpy as np
 import astropy.units as u
-from astropy.table import Table
 from astropy.coordinates import Angle
 from astropy.modeling.models import Gaussian1D
-from ..utils.scripts import make_path
-from ..utils.table import table_row_to_dict
-from ..utils.interpolation import ScaledRegularGridInterpolator
-from ..spectrum import FluxPoints
-from ..spectrum.models import PowerLaw, ExponentialCutoffPowerLaw
-from ..image.models import SkyPointSource, SkyGaussian, SkyShell
-from ..cube.models import SkyModel, SkyModels
+from astropy.table import Table
+from gammapy.modeling.models import (
+    ExpCutoffPowerLawSpectralModel,
+    GaussianSpatialModel,
+    PointSpatialModel,
+    PowerLawSpectralModel,
+    ShellSpatialModel,
+    SkyModel,
+    SkyModels,
+)
+from gammapy.spectrum import FluxPoints
+from gammapy.utils.interpolation import ScaledRegularGridInterpolator
+from gammapy.utils.scripts import make_path
+from gammapy.utils.table import table_row_to_dict
 from .core import SourceCatalog, SourceCatalogObject
 
 __all__ = [
@@ -44,10 +49,10 @@ class SourceCatalogObjectHGPSComponent:
     _source_index_key = "row_index"
 
     def __init__(self, data):
-        self.data = OrderedDict(data)
+        self.data = data
 
     def __repr__(self):
-        return "{}({!r})".format(self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}({self.name!r})"
 
     def __str__(self):
         """Pretty-print source data"""
@@ -78,9 +83,11 @@ class SourceCatalogObjectHGPSComponent:
 
     @property
     def spatial_model(self):
-        """Component spatial model (`~gammapy.image.models.SkyGaussian`)."""
+        """Component spatial model (`~gammapy.modeling.models.GaussianSpatialModel`)."""
         d = self.data
-        model = SkyGaussian(lon_0=d["GLON"], lat_0=d["GLAT"], sigma=d["Size"])
+        model = GaussianSpatialModel(
+            lon_0=d["GLON"], lat_0=d["GLAT"], sigma=d["Size"], frame="galactic"
+        )
         model.parameters.set_parameter_errors(
             dict(lon_0=d["GLON_Err"], lat_0=d["GLAT_Err"], sigma=d["Size_Err"])
         )
@@ -95,7 +102,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
     """
 
     def __repr__(self):
-        return "{}({!r})".format(self.__class__.__name__, self.name)
+        return f"{self.__class__.__name__}({self.name!r})"
 
     def __str__(self):
         return self.info()
@@ -151,9 +158,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
     def _info_id(self):
         ss = "\n*** Source identification info ***\n\n"
-        ss += "\n".join(
-            "{}: {}".format(k, v) for k, v in self.identification_info.items()
-        )
+        ss += "\n".join(f"{k}: {v}" for k, v in self.identification_info.items())
         return ss + "\n"
 
     def _info_map(self):
@@ -252,12 +257,12 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
             "Flux correction (RSpec -> Total)",
             100 * d["Flux_Correction_RSpec_To_Total"],
         )
-        ss += "{:<35s} : {:5.1f} %\n".format(label, val)
+        ss += f"{label:<35s} : {val:5.1f} %\n"
         label, val = (
             "Flux correction (Total -> RSpec)",
             100 * (1 / d["Flux_Correction_RSpec_To_Total"]),
         )
-        ss += "{:<35s} : {:5.1f} %\n".format(label, val)
+        ss += f"{label:<35s} : {val:5.1f} %\n"
 
         return ss
 
@@ -429,12 +434,12 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
         return self.data["Spectral_Model"].strip().lower()
 
     def spectral_model(self, which="best"):
-        """Spectral model (`~gammapy.spectrum.models.SpectralModel`).
+        """Spectral model (`~gammapy.modeling.models.SpectralModel`).
 
         One of the following models (given by ``Spectral_Model`` in the catalog):
 
-        - ``PL`` : `~gammapy.spectrum.models.PowerLaw`
-        - ``ECPL`` : `~gammapy.spectrum.models.ExponentialCutoffPowerLaw`
+        - ``PL`` : `~gammapy.modeling.models.PowerLawSpectralModel`
+        - ``ECPL`` : `~gammapy.modeling.models.ExpCutoffPowerLawSpectralModel`
 
         Parameters
         ----------
@@ -448,7 +453,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
         elif which in {"pl", "ecpl"}:
             spec_type = which
         else:
-            raise ValueError("Invalid selection: which = {!r}".format(which))
+            raise ValueError(f"Invalid selection: which = {which!r}")
 
         pars, errs = {}, {}
 
@@ -458,7 +463,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
             pars["reference"] = data["Energy_Spec_PL_Pivot"]
             errs["amplitude"] = data["Flux_Spec_PL_Diff_Pivot_Err"]
             errs["index"] = data["Index_Spec_PL_Err"]
-            model = PowerLaw(**pars)
+            model = PowerLawSpectralModel(**pars)
         elif spec_type == "ecpl":
             pars["index"] = data["Index_Spec_ECPL"]
             pars["amplitude"] = data["Flux_Spec_ECPL_Diff_Pivot"]
@@ -467,9 +472,9 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
             errs["index"] = data["Index_Spec_ECPL_Err"]
             errs["amplitude"] = data["Flux_Spec_ECPL_Diff_Pivot_Err"]
             errs["lambda_"] = data["Lambda_Spec_ECPL_Err"]
-            model = ExponentialCutoffPowerLaw(**pars)
+            model = ExpCutoffPowerLawSpectralModel(**pars)
         else:
-            raise ValueError("Invalid spectral model: {}".format(spec_type))
+            raise ValueError(f"Invalid spectral model: {spec_type}")
 
         model.parameters.set_parameter_errors(errs)
         return model
@@ -492,14 +497,14 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
     @property
     def spatial_model(self):
-        """Spatial model (`~gammapy.image.models.SkySpatialModel`).
+        """Spatial model (`~gammapy.modeling.models.SpatialModel`).
 
         One of the following models (given by ``Spatial_Model`` in the catalog):
 
-        - ``Point-Like`` or has a size upper limit : `~gammapy.image.models.SkyPointSource`
-        - ``Gaussian``: `~gammapy.image.models.SkyGaussian`
+        - ``Point-Like`` or has a size upper limit : `~gammapy.modeling.models.PointSpatialModel`
+        - ``Gaussian``: `~gammapy.modeling.models.GaussianSpatialModel`
         - ``2-Gaussian`` or ``3-Gaussian``: composite model (using ``+`` with Gaussians)
-        - ``Shell``: `~gammapy.image.models.SkyShell`
+        - ``Shell``: `~gammapy.modeling.models.ShellSpatialModel`
         """
         d = self.data
         glon = d["GLON"]
@@ -508,9 +513,11 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
         spatial_type = self.spatial_model_type
 
         if self.is_pointlike:
-            model = SkyPointSource(lon_0=glon, lat_0=glat)
+            model = PointSpatialModel(lon_0=glon, lat_0=glat, frame="galactic")
         elif spatial_type == "gaussian":
-            model = SkyGaussian(lon_0=glon, lat_0=glat, sigma=d["Size"])
+            model = GaussianSpatialModel(
+                lon_0=glon, lat_0=glat, sigma=d["Size"], frame="galactic"
+            )
         elif spatial_type in {"2-gaussian", "3-gaussian"}:
             raise ValueError("For Gaussian or Multi-Gaussian models, use sky_model()!")
         elif spatial_type == "shell":
@@ -519,9 +526,11 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
             r_out = d["Size"]
             radius = 0.95 * r_out
             width = r_out - radius
-            model = SkyShell(lon_0=glon, lat_0=glat, width=width, radius=radius)
+            model = ShellSpatialModel(
+                lon_0=glon, lat_0=glat, width=width, radius=radius, frame="galactic"
+            )
         else:
-            raise ValueError("Not a valid spatial model: {}".format(spatial_type))
+            raise ValueError(f"Not a valid spatial model: {spatial_type}")
 
         return model
 
@@ -535,7 +544,7 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
 
         Returns
         -------
-        sky_model : `~gammapy.cube.models.SkyModel`
+        sky_model : `~gammapy.modeling.models.SkyModel`
             Sky model of the catalog object.
         """
         if self.spatial_model_type in {"2-gaussian", "3-gaussian"}:
@@ -547,13 +556,19 @@ class SourceCatalogObjectHGPS(SourceCatalogObject):
                 spectral_model_comp = spectral_model.copy()
                 # weight amplitude of the component
                 spectral_model_comp.parameters["amplitude"].value *= weight
-                models.append(SkyModel(component.spatial_model, spectral_model_comp))
+                models.append(
+                    SkyModel(
+                        component.spatial_model,
+                        spectral_model_comp,
+                        name=component.name,
+                    )
+                )
 
             return SkyModels(models)
         else:
             spatial_model = self.spatial_model
             spectral_model = self.spectral_model(which=which)
-            return SkyModel(spatial_model, spectral_model)
+            return SkyModel(spatial_model, spectral_model, name=self.name)
 
     @property
     def flux_points(self):
@@ -638,6 +653,7 @@ class SourceCatalogHGPS(SourceCatalog):
 
         self._table_components = Table.read(filename, hdu="HGPS_GAUSS_COMPONENTS")
         self._table_associations = Table.read(filename, hdu="HGPS_ASSOCIATIONS")
+        self._table_associations["Separation"].format = ".6f"
         self._table_identifications = Table.read(filename, hdu="HGPS_IDENTIFICATIONS")
         self._table_large_scale_component = Table.read(
             filename, hdu="HGPS_LARGE_SCALE_COMPONENT"
@@ -749,7 +765,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Surface_Brightness", glon)
@@ -759,7 +775,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Surface_Brightness_Err", glon)
@@ -769,7 +785,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Width", glon)
@@ -779,7 +795,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("Width_Err", glon)
@@ -789,7 +805,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("GLAT", glon)
@@ -799,7 +815,7 @@ class SourceCatalogLargeScaleHGPS:
 
         Parameters
         ----------
-        glon : `~astropy.coordinates.Longitude` or `~astropy.coordinates.Angle`
+        glon : `~astropy.coordinates.Angle`
             Galactic Longitude.
         """
         return self._interpolate_parameter("GLAT_Err", glon)
