@@ -1,8 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import pytest
-from ...utils.testing import requires_data
-from ..registry import SourceCatalogRegistry
-from ..gammacat import SourceCatalogGammaCat
+from gammapy.catalog import SourceCatalogGammaCat, SourceCatalogRegistry
+from gammapy.utils.testing import requires_data
 from .test_core import make_test_catalog
 
 
@@ -14,13 +13,13 @@ def source_catalogs():
     return cats
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_info_table(source_catalogs):
     table = source_catalogs.info_table
     assert table.colnames == ["name", "description", "sources"]
 
 
-@requires_data("gammapy-data")
+@requires_data()
 def test_getitem(source_catalogs):
     cat = source_catalogs["2fhl"]
     assert cat.name == "2fhl"
