@@ -2,12 +2,12 @@
 import copy
 import logging
 import numpy as np
-from scipy.signal import fftconvolve
-from scipy.ndimage import label
+import scipy.ndimage
+import scipy.signal
+from astropy.convolution import Gaussian2DKernel, MexicanHat2DKernel
 from astropy.io import fits
 from astropy.table import Table
-from astropy.convolution import Gaussian2DKernel, MexicanHat2DKernel
-from ..maps import WcsNDMap, MapAxis, WcsGeom
+from gammapy.maps import MapAxis, WcsGeom, WcsNDMap
 
 __all__ = ["CWT", "CWTData", "CWTKernels"]
 
@@ -75,7 +75,7 @@ class CWT:
     References
     ----------
     R. Terrier et al (2001) "Wavelet analysis of EGRET data"
-    See http://adsabs.harvard.edu/abs/2001ICRC....7.2923T
+    See https://ui.adsabs.harvard.edu/abs/2001ICRC....7.2923T
     """
 
     def __init__(
@@ -125,29 +125,31 @@ class CWT:
         """
         total_background = data._model + data._background + data._approx
         excess = data._counts - total_background
-        log.debug("Excess sum: {:.4f}".format(excess.sum()))
-        log.debug("Excess max: {:.4f}".format(excess.max()))
+        log.debug(f"Excess sum: {excess.sum():.4f}")
+        log.debug(f"Excess max: {excess.max():.4f}")
 
         log.debug("Computing transform and error")
         for idx_scale, kern in self.kernels.kern_base.items():
-            data._transform_3d[idx_scale] = fftconvolve(excess, kern, mode="same")
-            data._error[idx_scale] = np.sqrt(
-                fftconvolve(total_background, kern ** 2, mode="same")
+            data._transform_3d[idx_scale] = scipy.signal.fftconvolve(
+                excess, kern, mode="same"
             )
-        log.debug("Error sum: {:.4f}".format(data._error.sum()))
-        log.debug("Error max: {:.4f}".format(data._error.max()))
+            data._error[idx_scale] = np.sqrt(
+                scipy.signal.fftconvolve(total_background, kern ** 2, mode="same")
+            )
+        log.debug(f"Error sum: {data._error.sum():.4f}")
+        log.debug(f"Error max: {data._error.max():.4f}")
 
         log.debug("Computing approx and approx_bkg")
-        data._approx = fftconvolve(
+        data._approx = scipy.signal.fftconvolve(
             data._counts - data._model - data._background,
             self.kernels.kern_approx,
             mode="same",
         )
-        data._approx_bkg = fftconvolve(
+        data._approx_bkg = scipy.signal.fftconvolve(
             data._background, self.kernels.kern_approx, mode="same"
         )
-        log.debug("Approximate sum: {:.4f}".format(data._approx.sum()))
-        log.debug("Approximate background sum: {:.4f}".format(data._approx_bkg.sum()))
+        log.debug(f"Approximate sum: {data._approx.sum():.4f}")
+        log.debug(f"Approximate background sum: {data._approx_bkg.sum():.4f}")
 
     def _compute_support(self, data):
         """Compute the multiresolution support with hard sigma clipping.
@@ -191,7 +193,7 @@ class CWT:
             mask = significance[idx_scale] > self.significance_threshold
 
             # Produce a list of connex structures in the support
-            labeled_mask, n_structures = label(mask)
+            labeled_mask, n_structures = scipy.ndimage.label(mask)
             for struct_label in range(n_structures):
                 coords = np.where(labeled_mask == struct_label + 1)
 
@@ -264,7 +266,7 @@ class CWT:
         variance_ratio = abs(
             (self.previous_variance - variance) / self.previous_variance
         )
-        log.info("Variance ratio: {:.7f}".format(variance_ratio))
+        log.info(f"Variance ratio: {variance_ratio:.7f}")
 
         self.previous_variance = variance
         return variance_ratio < self.tol
@@ -283,7 +285,7 @@ class CWT:
             self.history = [copy.deepcopy(data)]
 
         for n_iter in range(self.max_iter):
-            log.info("************ Start iteration {} ************".format(n_iter + 1))
+            log.info(f"************ Start iteration {n_iter + 1} ************")
             self._execute_iteration(data=data)
             if self.history is not None:
                 log.debug("Save current data")
@@ -293,11 +295,9 @@ class CWT:
                 break
 
         if converge_answer:
-            log.info("Convergence reached at iteration {}".format(n_iter + 1))
+            log.info(f"Convergence reached at iteration {n_iter + 1}")
         else:
-            log.info(
-                "Convergence not formally reached at iteration {}".format(n_iter + 1)
-            )
+            log.info(f"Convergence not formally reached at iteration {n_iter + 1}")
 
 
 class CWTKernels:
@@ -394,13 +394,13 @@ class CWTKernels:
         info_dict["Kernels approx max"] = self.kern_approx.max()
 
         for idx_scale, scale in enumerate(self.scales):
-            info_dict["Kernels base width for {} scale".format(scale)] = len(
+            info_dict[f"Kernels base width for {scale} scale"] = len(
                 self.kern_base[idx_scale]
             )
-            info_dict["Kernels base sum for {} scale".format(scale)] = self.kern_base[
+            info_dict[f"Kernels base sum for {scale} scale"] = self.kern_base[
                 idx_scale
             ].sum()
-            info_dict["Kernels base max for {} scale".format(scale)] = self.kern_base[
+            info_dict[f"Kernels base max for {scale} scale"] = self.kern_base[
                 idx_scale
             ].max()
 
