@@ -1,12 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from collections import OrderedDict
 import numpy as np
 import astropy.units as u
 from astropy.io import fits
 from astropy.table import Table
-from ..utils.nddata import NDDataArray, BinnedDataAxis
-from ..utils.energy import EnergyBounds
-from ..utils.scripts import make_path
+from gammapy.maps import MapAxis
+from gammapy.maps.utils import edges_from_lo_hi
+from gammapy.utils.energy import energy_logcenter
+from gammapy.utils.nddata import NDDataArray
+from gammapy.utils.scripts import make_path
 
 __all__ = ["EffectiveAreaTable", "EffectiveAreaTable2D"]
 
@@ -62,14 +63,15 @@ class EffectiveAreaTable:
     """
 
     def __init__(self, energy_lo, energy_hi, data, meta=None):
-        axes = [
-            BinnedDataAxis(
-                energy_lo, energy_hi, interpolation_mode="log", name="energy"
-            )
-        ]
+
+        e_edges = edges_from_lo_hi(energy_lo, energy_hi)
+        energy_axis = MapAxis.from_edges(e_edges, interp="log", name="energy")
+
         interp_kwargs = {"extrapolate": False, "bounds_error": False}
-        self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+        self.data = NDDataArray(
+            axes=[energy_axis], data=data, interp_kwargs=interp_kwargs
+        )
+        self.meta = meta or {}
 
     @property
     def energy(self):
@@ -99,32 +101,35 @@ class EffectiveAreaTable:
         kwargs.setdefault("lw", 2)
 
         if energy is None:
-            energy = self.energy.nodes
+            energy = self.energy.center
+
         eff_area = self.data.evaluate(energy=energy)
+
         xerr = (
-            energy.value - self.energy.lo.value,
-            self.energy.hi.value - energy.value,
+            (energy - self.energy.edges[:-1]).value,
+            (self.energy.edges[1:] - energy).value,
         )
+
         ax.errorbar(energy.value, eff_area.value, xerr=xerr, **kwargs)
         if show_energy is not None:
             ener_val = u.Quantity(show_energy).to_value(self.energy.unit)
             ax.vlines(ener_val, 0, 1.1 * self.max_area.value, linestyles="dashed")
         ax.set_xscale("log")
-        ax.set_xlabel("Energy [{}]".format(self.energy.unit))
-        ax.set_ylabel("Effective Area [{}]".format(self.data.data.unit))
+        ax.set_xlabel(f"Energy [{self.energy.unit}]")
+        ax.set_ylabel(f"Effective Area [{self.data.data.unit}]")
 
         return ax
 
     @classmethod
     def from_parametrization(cls, energy, instrument="HESS"):
-        """Get parametrized effective area.
+        r"""Get parametrized effective area.
 
         Parametrizations of the effective areas of different Cherenkov
         telescopes taken from Appendix B of Abramowski et al. (2010), see
-        http://adsabs.harvard.edu/abs/2010MNRAS.402.1342A .
+        https://ui.adsabs.harvard.edu/abs/2010MNRAS.402.1342A .
 
         .. math::
-            A_{eff}(E) = g_1 \\left(\\frac{E}{\\mathrm{MeV}}\\right)^{-g_2}\\exp{\\left(-\\frac{g_3}{E}\\right)}
+            A_{eff}(E) = g_1 \left(\frac{E}{\mathrm{MeV}}\right)^{-g_2}\exp{\left(-\frac{g_3}{E}\right)}
 
         Parameters
         ----------
@@ -133,7 +138,7 @@ class EffectiveAreaTable:
         instrument : {'HESS', 'HESS2', 'CTA'}
             Instrument name
         """
-        energy = EnergyBounds(energy)
+        energy = u.Quantity(energy)
         # Put the parameters g in a dictionary.
         # Units: g1 (cm^2), g2 (), g3 (MeV)
         # Note that whereas in the paper the parameter index is 1-based,
@@ -145,11 +150,11 @@ class EffectiveAreaTable:
         }
 
         if instrument not in pars.keys():
-            ss = "Unknown instrument: {}\n".format(instrument)
+            ss = f"Unknown instrument: {instrument}\n"
             ss += "Valid instruments: HESS, HESS2, CTA"
             raise ValueError(ss)
 
-        xx = energy.log_centers.to_value("MeV")
+        xx = energy_logcenter(energy).to_value("MeV")
 
         g1 = pars[instrument][0]
         g2 = pars[instrument][1]
@@ -158,9 +163,7 @@ class EffectiveAreaTable:
         value = g1 * xx ** (-g2) * np.exp(g3 / xx)
         data = u.Quantity(value, "cm2", copy=False)
 
-        return cls(
-            energy_lo=energy.lower_bounds, energy_hi=energy.upper_bounds, data=data
-        )
+        return cls(energy_lo=energy[:-1], energy_hi=energy[1:], data=data)
 
     @classmethod
     def from_table(cls, table):
@@ -186,9 +189,10 @@ class EffectiveAreaTable:
             try:
                 aeff = cls.from_hdulist(hdulist, hdu=hdu)
             except KeyError:
-                msg = 'File {} contains no HDU "{}"'.format(filename, hdu)
-                msg += "\n Available {}".format([_.name for _ in hdulist])
-                raise ValueError(msg)
+                raise ValueError(
+                    f"File {filename} contains no HDU {hdu!r}\n"
+                    f"Available: {[_.name for _ in hdulist]}"
+                )
 
         return aeff
 
@@ -198,29 +202,34 @@ class EffectiveAreaTable:
         Data format specification: :ref:`gadf:ogip-arf`
         """
         table = Table()
-        table.meta = OrderedDict(
-            [
-                ("EXTNAME", "SPECRESP"),
-                ("hduclass", "OGIP"),
-                ("hduclas1", "RESPONSE"),
-                ("hduclas2", "SPECRESP"),
-            ]
-        )
-        table["ENERG_LO"] = self.energy.lo
-        table["ENERG_HI"] = self.energy.hi
+        table.meta = {
+            "EXTNAME": "SPECRESP",
+            "hduclass": "OGIP",
+            "hduclas1": "RESPONSE",
+            "hduclas2": "SPECRESP",
+        }
+
+        energy = self.energy.edges
+        table["ENERG_LO"] = energy[:-1]
+        table["ENERG_HI"] = energy[1:]
         table["SPECRESP"] = self.evaluate_fill_nan()
         return table
 
-    def to_hdulist(self, name=None):
+    def to_hdulist(self, name=None, use_sherpa=False):
         """Convert to `~astropy.io.fits.HDUList`."""
-        return fits.HDUList(
-            [fits.PrimaryHDU(), fits.BinTableHDU(self.to_table(), name=name)]
-        )
+        table = self.to_table()
 
-    def write(self, filename, **kwargs):
+        if use_sherpa:
+            table["ENERG_HI"] = table["ENERG_HI"].quantity.to("keV")
+            table["ENERG_LO"] = table["ENERG_LO"].quantity.to("keV")
+            table["SPECRESP"] = table["SPECRESP"].quantity.to("cm2")
+
+        return fits.HDUList([fits.PrimaryHDU(), fits.BinTableHDU(table, name=name)])
+
+    def write(self, filename, use_sherpa=False, **kwargs):
         """Write to file."""
         filename = make_path(filename)
-        self.to_hdulist().writeto(str(filename), **kwargs)
+        self.to_hdulist(use_sherpa=use_sherpa).writeto(str(filename), **kwargs)
 
     def evaluate_fill_nan(self, **kwargs):
         """Modified evaluate function.
@@ -265,9 +274,9 @@ class EffectiveAreaTable:
         energy : `~astropy.units.Quantity`
             Energy corresponding to the given aeff.
         """
-        from ..spectrum.models import TableModel
+        from gammapy.modeling.models import TemplateSpectralModel
 
-        energy = self.energy.nodes
+        energy = self.energy.center
 
         if emin is None:
             emin = energy[0]
@@ -275,7 +284,7 @@ class EffectiveAreaTable:
             # use the peak effective area as a default for the energy maximum
             emax = energy[np.argmax(self.data.data)]
 
-        aeff_spectrum = TableModel(energy, self.data.data, values_scale="lin")
+        aeff_spectrum = TemplateSpectralModel(energy, self.data.data)
         return aeff_spectrum.inverse(aeff, emin=emin, emax=emax)
 
     def to_sherpa(self, name):
@@ -358,20 +367,25 @@ class EffectiveAreaTable2D:
 
         if interp_kwargs is None:
             interp_kwargs = self.default_interp_kwargs
-        axes = [
-            BinnedDataAxis(
-                energy_lo, energy_hi, interpolation_mode="log", name="energy"
-            ),
-            BinnedDataAxis(
-                offset_lo, offset_hi, interpolation_mode="linear", name="offset"
-            ),
-        ]
-        self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+
+        e_edges = edges_from_lo_hi(energy_lo, energy_hi)
+        energy_axis = MapAxis.from_edges(e_edges, interp="log", name="energy")
+
+        # TODO: for some reason the H.E.S.S. DL3 files contain the same values for offset_hi and offset_lo
+        if np.allclose(offset_lo.to_value("deg"), offset_hi.to_value("deg")):
+            offset_axis = MapAxis.from_nodes(offset_lo, interp="lin", name="offset")
+        else:
+            offset_edges = edges_from_lo_hi(offset_lo, offset_hi)
+            offset_axis = MapAxis.from_edges(offset_edges, interp="lin", name="offset")
+
+        self.data = NDDataArray(
+            axes=[energy_axis, offset_axis], data=data, interp_kwargs=interp_kwargs
+        )
+        self.meta = meta or {}
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += "\n{}".format(self.data)
+        ss += f"\n{self.data}"
         return ss
 
     @property
@@ -421,13 +435,12 @@ class EffectiveAreaTable2D:
             Energy axis bin edges
         """
         if energy is None:
-            energy = self.data.axis("energy").bins
+            energy = self.data.axis("energy").edges
 
-        energy = EnergyBounds(energy)
-        area = self.data.evaluate(offset=offset, energy=energy.log_centers)
+        area = self.data.evaluate(offset=offset, energy=energy_logcenter(energy))
 
         return EffectiveAreaTable(
-            energy_lo=energy.lower_bounds, energy_hi=energy.upper_bounds, data=area
+            energy_lo=energy[:-1], energy_hi=energy[1:], data=area
         )
 
     def plot_energy_dependence(self, ax=None, offset=None, energy=None, **kwargs):
@@ -454,20 +467,20 @@ class EffectiveAreaTable2D:
         ax = plt.gca() if ax is None else ax
 
         if offset is None:
-            off_min, off_max = self.data.axis("offset").nodes[[0, -1]].value
-            offset = np.linspace(off_min, off_max, 4) * self.data.axis("offset").unit
+            off_min, off_max = self.data.axis("offset").center[[0, -1]]
+            offset = np.linspace(off_min.value, off_max.value, 4) * off_min.unit
 
         if energy is None:
-            energy = self.data.axis("energy").nodes
+            energy = self.data.axis("energy").center
 
         for off in offset:
             area = self.data.evaluate(offset=off, energy=energy)
-            label = "offset = {:.1f}".format(off)
+            label = f"offset = {off:.1f}"
             ax.plot(energy, area.value, label=label, **kwargs)
 
         ax.set_xscale("log")
-        ax.set_xlabel("Energy [{}]".format(self.data.axis("energy").unit))
-        ax.set_ylabel("Effective Area [{}]".format(self.data.data.unit))
+        ax.set_xlabel(f"Energy [{self.data.axis('energy').unit}]")
+        ax.set_ylabel(f"Effective Area [{self.data.data.unit}]")
         ax.set_xlim(min(energy.value), max(energy.value))
         ax.legend(loc="upper left")
 
@@ -482,7 +495,7 @@ class EffectiveAreaTable2D:
             Axis
         offset : `~astropy.coordinates.Angle`
             Offset axis
-        energy : `~gammapy.utils.energy.Energy`
+        energy : `~astropy.units.Quantity`
             Energy
 
         Returns
@@ -495,23 +508,22 @@ class EffectiveAreaTable2D:
         ax = plt.gca() if ax is None else ax
 
         if energy is None:
-            e_min, e_max = np.log10(self.data.axis("energy").nodes[[0, -1]].value)
+            e_min, e_max = np.log10(self.data.axis("energy").center.value[[0, -1]])
             energy = np.logspace(e_min, e_max, 4) * self.data.axis("energy").unit
 
         if offset is None:
-            off_lo, off_hi = self.data.axis("offset").nodes[[0, -1]].to_value("deg")
-            offset = np.linspace(off_lo, off_hi, 100) * u.deg
+            offset = self.data.axis("offset").center
 
         for ee in energy:
             area = self.data.evaluate(offset=offset, energy=ee)
             area /= np.nanmax(area)
             if np.isnan(area).all():
                 continue
-            label = "energy = {:.1f}".format(ee)
+            label = f"energy = {ee:.1f}"
             ax.plot(offset, area, label=label, **kwargs)
 
         ax.set_ylim(0, 1.1)
-        ax.set_xlabel("Offset ({})".format(self.data.axis("offset").unit))
+        ax.set_xlabel(f"Offset ({self.data.axis('offset').unit})")
         ax.set_ylabel("Relative Effective Area")
         ax.legend(loc="best")
 
@@ -523,8 +535,8 @@ class EffectiveAreaTable2D:
 
         ax = plt.gca() if ax is None else ax
 
-        offset = self.data.axis("offset").bins
-        energy = self.data.axis("energy").bins
+        energy = self.data.axis("energy").edges
+        offset = self.data.axis("offset").edges
         aeff = self.data.evaluate(offset=offset, energy=energy[:, np.newaxis])
 
         vmin, vmax = np.nanmin(aeff.value), np.nanmax(aeff.value)
@@ -537,14 +549,14 @@ class EffectiveAreaTable2D:
         caxes = ax.pcolormesh(energy.value, offset.value, aeff.value.T, **kwargs)
 
         ax.set_xscale("log")
-        ax.set_ylabel("Offset ({})".format(offset.unit))
-        ax.set_xlabel("Energy ({})".format(energy.unit))
+        ax.set_ylabel(f"Offset ({offset.unit})")
+        ax.set_xlabel(f"Energy ({energy.unit})")
 
         xmin, xmax = energy.value.min(), energy.value.max()
         ax.set_xlim(xmin, xmax)
 
         if add_cbar:
-            label = "Effective Area ({unit})".format(unit=aeff.unit)
+            label = f"Effective Area ({aeff.unit})"
             ax.figure.colorbar(caxes, ax=ax, label=label)
 
         return ax
@@ -562,14 +574,18 @@ class EffectiveAreaTable2D:
     def to_table(self):
         """Convert to `~astropy.table.Table`."""
         meta = self.meta.copy()
+
+        energy = self.data.axis("energy").edges
+        theta = self.data.axis("offset").edges
+
         table = Table(meta=meta)
-        table["ENERG_LO"] = self.data.axis("energy").lo[np.newaxis]
-        table["ENERG_HI"] = self.data.axis("energy").hi[np.newaxis]
-        table["THETA_LO"] = self.data.axis("offset").lo[np.newaxis]
-        table["THETA_HI"] = self.data.axis("offset").hi[np.newaxis]
+        table["ENERG_LO"] = energy[:-1][np.newaxis]
+        table["ENERG_HI"] = energy[1:][np.newaxis]
+        table["THETA_LO"] = theta[:-1][np.newaxis]
+        table["THETA_HI"] = theta[1:][np.newaxis]
         table["EFFAREA"] = self.data.data.T[np.newaxis]
         return table
 
     def to_fits(self, name="EFFECTIVE AREA"):
-        """Convert to `~astropy.io.fits.BinTable`."""
+        """Convert to `~astropy.io.fits.BinTableHDU`."""
         return fits.BinTableHDU(self.to_table(), name=name)
