@@ -1,17 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from collections import OrderedDict
 import numpy as np
-from scipy.special import erf
-from astropy.io import fits
+import scipy.special
 from astropy.coordinates import Angle
-from astropy.units import Quantity
+from astropy.io import fits
 from astropy.table import Table
-from ..maps import MapAxis
-from ..maps.utils import edges_from_lo_hi
-from ..utils.scripts import make_path
-from ..utils.nddata import NDDataArray
-from ..utils.fits import energy_axis_to_ebounds
-from ..utils.energy import energy_logcenter
+from astropy.units import Quantity
+from gammapy.maps import MapAxis
+from gammapy.maps.utils import edges_from_lo_hi
+from gammapy.utils.energy import energy_logcenter
+from gammapy.utils.fits import energy_axis_to_ebounds
+from gammapy.utils.nddata import NDDataArray
+from gammapy.utils.scripts import make_path
 
 __all__ = ["EnergyDispersion", "EnergyDispersion2D"]
 
@@ -79,11 +78,11 @@ class EnergyDispersion:
         self.data = NDDataArray(
             axes=[e_true_axis, e_reco_axis], data=data, interp_kwargs=interp_kwargs
         )
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+        self.meta = meta or {}
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += "\n{}".format(self.data)
+        ss += f"\n{self.data}"
         return ss
 
     def apply(self, data):
@@ -105,9 +104,7 @@ class EnergyDispersion:
         """
         if len(data) != self.e_true.nbin:
             raise ValueError(
-                "Input size {} does not match true energy axis {}".format(
-                    len(data), self.e_true.nbin
-                )
+                f"Input size {len(data)} does not match true energy axis {self.e_true.nbin}"
             )
         return np.dot(data, self.data.data)
 
@@ -255,9 +252,9 @@ class EnergyDispersion:
                 for k in range(l.field("N_GRP")):
                     pdf_matrix[
                         i,
-                        l.field("F_CHAN")[k]: l.field("F_CHAN")[k]
+                        l.field("F_CHAN")[k] : l.field("F_CHAN")[k]
                         + l.field("N_CHAN")[k],
-                    ] = l.field("MATRIX")[m_start: m_start + l.field("N_CHAN")[k]]
+                    ] = l.field("MATRIX")[m_start : m_start + l.field("N_CHAN")[k]]
                     m_start += l.field("N_CHAN")[k]
 
         unit = ebounds_hdu.header.get("TUNIT2")
@@ -330,10 +327,10 @@ class EnergyDispersion:
 
         cols = table.columns
         c0 = fits.Column(
-            name=cols[0].name, format="E", array=cols[0], unit="{}".format(cols[0].unit)
+            name=cols[0].name, format="E", array=cols[0], unit=str(cols[0].unit)
         )
         c1 = fits.Column(
-            name=cols[1].name, format="E", array=cols[1], unit="{}".format(cols[1].unit)
+            name=cols[1].name, format="E", array=cols[1], unit=str(cols[1].unit)
         )
         c2 = fits.Column(name=cols[2].name, format="I", array=cols[2])
         c3 = fits.Column(name=cols[3].name, format="PI()", array=cols[3])
@@ -402,19 +399,17 @@ class EnergyDispersion:
         table["N_CHAN"] = n_chan
         table["MATRIX"] = matrix
 
-        table.meta = OrderedDict(
-            [
-                ("name", "MATRIX"),
-                ("chantype", "PHA"),
-                ("hduclass", "OGIP"),
-                ("hduclas1", "RESPONSE"),
-                ("hduclas2", "RSP_MATRIX"),
-                ("detchans", self.e_reco.nbin),
-                ("numgrp", numgrp),
-                ("numelt", numelt),
-                ("tlmin4", 0),
-            ]
-        )
+        table.meta = {
+            "name": "MATRIX",
+            "chantype": "PHA",
+            "hduclass": "OGIP",
+            "hduclas1": "RESPONSE",
+            "hduclas2": "RSP_MATRIX",
+            "detchans": self.e_reco.nbin,
+            "numgrp": numgrp,
+            "numelt": numelt,
+            "tlmin4": 0,
+        }
 
         return table
 
@@ -459,9 +454,9 @@ class EnergyDispersion:
     def get_bias_energy(self, bias, emin=None, emax=None):
         """Find energy corresponding to a given bias.
 
-        In case the solution is not unique, provide the `emin` or `emax` arguments
+        In case the solution is not unique, provide the ``emin`` or ``emax`` arguments
         to limit the solution to the given range.  By default the peak energy of the
-        bias is chosen as `emin`.
+        bias is chosen as ``emin``.
 
         Parameters
         ----------
@@ -474,10 +469,10 @@ class EnergyDispersion:
 
         Returns
         -------
-        bias_energy : `~astropy.unit.Quantity`
+        bias_energy : `~astropy.units.Quantity`
             Reconstructed energy corresponding to the given bias.
         """
-        from ..spectrum.models import TableModel
+        from gammapy.modeling.models import TemplateSpectralModel
 
         e_true = self.e_true.center
         values = self.get_bias(e_true)
@@ -488,7 +483,7 @@ class EnergyDispersion:
         if emax is None:
             emax = e_true[-1]
 
-        bias_spectrum = TableModel(e_true, values)
+        bias_spectrum = TemplateSpectralModel(e_true, values)
         e_true_bias = bias_spectrum.inverse(Quantity(bias), emin=emin, emax=emax)
 
         # return reconstructed energy
@@ -627,8 +622,8 @@ class EnergyDispersion:
             label = "Probability density (A.U.)"
             cbar = ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
-        ax.set_ylabel(r"$E_\mathrm{{Reco}}$ [{unit}]".format(unit=e_reco.unit))
+        ax.set_xlabel(fr"$E_\mathrm{{True}}$ [{e_true.unit}]")
+        ax.set_ylabel(fr"$E_\mathrm{{Reco}}$ [{e_reco.unit}]")
         ax.set_xscale("log")
         ax.set_yscale("log")
         ax.set_xlim(x.min(), x.max())
@@ -753,11 +748,11 @@ class EnergyDispersion2D:
         axes = [e_true_axis, migra_axis, offset_axis]
 
         self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
-        self.meta = OrderedDict(meta) if meta else OrderedDict()
+        self.meta = meta or {}
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += "\n{}".format(self.data)
+        ss += f"\n{self.data}"
         return ss
 
     @classmethod
@@ -800,7 +795,7 @@ class EnergyDispersion2D:
         s = np.sqrt(2) * sigma
         t1 = (migra2d_hi - 1 - bias) / s
         t2 = (migra2d_lo - 1 - bias) / s
-        pdf = (erf(t1) - erf(t2)) / 2
+        pdf = (scipy.special.erf(t1) - scipy.special.erf(t2)) / 2
 
         pdf_array = pdf.T[:, :, np.newaxis] * np.ones(len(offset) - 1)
 
@@ -975,9 +970,9 @@ class EnergyDispersion2D:
             Axis
         offset : `~astropy.coordinates.Angle`, optional
             Offset
-        e_true : `~astropy.utils.Quantity`, optional
+        e_true : `~astropy.units.Quantity`, optional
             True energy
-        migra : `~numpy.array`, list, optional
+        migra : `~numpy.ndarray`, optional
             Migration nodes
 
         Returns
@@ -1004,7 +999,7 @@ class EnergyDispersion2D:
         for ener in e_true:
             for off in offset:
                 disp = self.data.evaluate(offset=off, e_true=ener, migra=migra)
-                label = "offset = {:.1f}\nenergy = {:.1f}".format(off, ener)
+                label = f"offset = {off:.1f}\nenergy = {ener:.1f}"
                 ax.plot(migra, disp, label=label, **kwargs)
 
         ax.set_xlabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
@@ -1060,7 +1055,7 @@ class EnergyDispersion2D:
             label = "Probability density (A.U.)"
             ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
+        ax.set_xlabel(fr"$E_\mathrm{{True}}$ [{e_true.unit}]")
         ax.set_ylabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
         ax.set_xlim(x.min(), x.max())
         ax.set_ylim(y.min(), y.max())
