@@ -1,11 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from numpy.testing import assert_allclose
-from ...utils.testing import requires_data
-from ..psf_3d import PSF3D
-from ..psf_check import PSF3DChecker
+from gammapy.irf import PSF3D
+from gammapy.irf.psf_check import PSF3DChecker
+from gammapy.utils.testing import requires_data
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestPSF3DChecker:
     def setup(self):
         filename = "$GAMMAPY_DATA/tests/hess-hap-hd-prod3/psf_table.fits.gz"
@@ -25,6 +25,6 @@ class TestPSF3DChecker:
         # Check results
         assert res["nan"]["status"] == "failed"
         assert res["nan"]["n_failed_bins"] == 4
-        assert res["normalise"]["status"] == "ok"
+        assert res["normalize"]["status"] == "ok"
         assert res["containment"]["status"] == "ok"
         assert res["status"] == "failed"
