@@ -1,17 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Utilities for dealing with HEALPix projections and mappings."""
-from collections import OrderedDict
-import re
 import copy
+import re
 import numpy as np
-from astropy.io import fits
 from astropy.coordinates import SkyCoord
+from astropy.io import fits
 from astropy.units import Quantity
+from .geom import (
+    Geom,
+    MapCoord,
+    coordsys_to_frame,
+    find_and_read_bands,
+    make_axes,
+    pix_tuple_to_idx,
+    skycoord_to_lonlat,
+)
 from .utils import INVALID_INDEX
 from .wcs import WcsGeom
-from .geom import MapGeom, MapCoord, pix_tuple_to_idx
-from .geom import coordsys_to_frame, skycoord_to_lonlat
-from .geom import find_and_read_bands, make_axes
 
 # Not sure if we should expose this in the docs or not:
 # HPX_FITS_CONVENTIONS, HpxConv
@@ -37,7 +42,7 @@ class HpxConv:
         self.coordsys = kwargs.get("coordsys", "COORDSYS")
 
     def colname(self, indx):
-        return "{}{}".format(self.colstring, indx)
+        return f"{self.colstring}{indx}"
 
     @classmethod
     def create(cls, convname="gadf"):
@@ -179,7 +184,7 @@ def get_pix_size_from_nside(nside):
     """
     order = nside_to_order(nside)
     if np.any(order < 0) or np.any(order > 13):
-        raise ValueError("HEALPIX order must be 0 to 13. Got: {!r}".format(order))
+        raise ValueError(f"HEALPIX order must be 0 to 13. Got: {order!r}")
 
     return HPX_ORDER_TO_PIXSIZE[order]
 
@@ -308,8 +313,7 @@ def make_hpx_to_wcs_mapping(hpx, wcs):
 
 
 def match_hpx_pix(nside, nest, nside_pix, ipix_ring):
-    """TODO
-    """
+    """TODO: document."""
     import healpy as hp
 
     ipix_in = np.arange(12 * nside * nside)
@@ -324,7 +328,7 @@ def parse_hpxregion(region):
     m = re.match(r"([A-Za-z\_]*?)\((.*?)\)", region)
 
     if m is None:
-        raise ValueError("Failed to parse hpx region string: {!r}".format(region))
+        raise ValueError(f"Failed to parse hpx region string: {region!r}")
 
     if not m.group(1):
         return re.split(",", m.group(2))
@@ -361,13 +365,13 @@ def get_hpxregion_dir(region, coordsys):
         elif tokens[1] == "RING":
             nest_pix = False
         else:
-            raise ValueError("Invalid ordering scheme: {!r}".format(tokens[1]))
+            raise ValueError(f"Invalid ordering scheme: {tokens[1]!r}")
         theta, phi = hp.pix2ang(nside_pix, ipix_pix, nest_pix)
         lat = np.degrees((np.pi / 2) - theta)
         lon = np.degrees(phi)
         return SkyCoord(lon, lat, frame=frame, unit="deg")
     else:
-        raise ValueError("Invalid region type: {!r}".format(tokens[0]))
+        raise ValueError(f"Invalid region type: {tokens[0]!r}")
 
 
 def get_hpxregion_size(region):
@@ -380,7 +384,7 @@ def get_hpxregion_size(region):
         pix_size = get_pix_size_from_nside(int(tokens[2]))
         return 2.0 * pix_size
     else:
-        raise ValueError("Invalid region type: {!r}".format(tokens[0]))
+        raise ValueError(f"Invalid region type: {tokens[0]!r}")
 
 
 def is_power2(n):
@@ -460,6 +464,7 @@ def get_superpixels(idx, nside_subpix, nside_superpix, nest=True):
 
 def get_subpixels(idx, nside_superpix, nside_subpix, nest=True):
     """Compute the indices of subpixels contained within superpixels.
+
     This function returns an output array with one additional
     dimension of size N for subpixel indices where N is the maximum
     number of subpixels for any pair of ``nside_superpix`` and
@@ -518,7 +523,7 @@ def get_subpixels(idx, nside_superpix, nside_subpix, nest=True):
     return idx
 
 
-class HpxGeom(MapGeom):
+class HpxGeom(Geom):
     """Geometry class for HEALPIX maps.
 
     This class performs mapping between partial-sky indices (pixel
@@ -552,8 +557,6 @@ class HpxGeom(MapGeom):
         encompassed by the geometry.
     axes : list
         Axes for non-spatial dimensions.
-    conv : str
-        Convention for FITS serialization format.
     sparse : bool
         If True defer allocation of partial- to all-sky index mapping
         arrays.  This option is only compatible with partial-sky maps
@@ -563,21 +566,14 @@ class HpxGeom(MapGeom):
     is_hpx = True
 
     def __init__(
-        self,
-        nside,
-        nest=True,
-        coordsys="CEL",
-        region=None,
-        axes=None,
-        conv="gadf",
-        sparse=False,
+        self, nside, nest=True, coordsys="CEL", region=None, axes=None, sparse=False
     ):
 
         # FIXME: Figure out what to do when sparse=True
         # FIXME: Require NSIDE to be power of two when nest=True
 
         self._nside = np.array(nside, ndmin=1)
-        self._axes = make_axes(axes, conv)
+        self._axes = make_axes(axes)
         if self.nside.size > 1 and self.nside.shape != self.shape_axes:
             raise ValueError(
                 "Wrong dimensionality for nside. nside must "
@@ -603,7 +599,6 @@ class HpxGeom(MapGeom):
                 self._rmap[ipix] = i
 
         self._npix = self._npix * np.ones(self.shape_axes, dtype=int)
-        self._conv = conv
         self._center_skydir = self._get_ref_dir()
         lon, lat, frame = skycoord_to_lonlat(self._center_skydir)
         self._center_coord = tuple(
@@ -665,17 +660,15 @@ class HpxGeom(MapGeom):
             self._npix = self._maxpix
 
         else:
-            raise ValueError("Invalid region string: {!r}".format(region))
+            raise ValueError(f"Invalid region string: {region!r}")
 
     def local_to_global(self, idx_local):
-        """Compute a local index (partial-sky) from a global (all-sky)
-        index.
+        """Compute a local index (partial-sky) from a global (all-sky) index.
 
         Returns
         -------
         idx_global : tuple
-            A tuple of pixel index vectors with global HEALPIX pixel
-            indices.
+            A tuple of pixel index vectors with global HEALPIX pixel indices
         """
         if self._ipix is None:
             return idx_local
@@ -886,7 +879,7 @@ class HpxGeom(MapGeom):
         else:
             region = self.region
 
-        return self.__class__(nside, self.nest, self.coordsys, region, axes, self.conv)
+        return self.__class__(nside, self.nest, self.coordsys, region, axes)
 
     @property
     def axes(self):
@@ -915,8 +908,9 @@ class HpxGeom(MapGeom):
 
     @property
     def order(self):
-        """ORDER in each band (NSIDE = 2**ORDER).  Set to -1 for bands with
-        NSIDE that is not a power of 2.
+        """ORDER in each band (``NSIDE = 2 ** ORDER``).
+
+        Set to -1 for bands with NSIDE that is not a power of 2.
         """
         return self._order
 
@@ -934,15 +928,6 @@ class HpxGeom(MapGeom):
         """
         return self._npix
 
-    @property
-    def conv(self):
-        """Name of default FITS convention associated with this geometry."""
-        return self._conv
-
-    @property
-    def hpx_conv(self):
-        return HPX_FITS_CONVENTIONS[self.conv]
-
     @property
     def coordsys(self):
         return self._coordsys
@@ -1003,8 +988,9 @@ class HpxGeom(MapGeom):
         return self.get_idx()
 
     def to_ud_graded(self, order):
-        """Upgrade or downgrade the resolution of this geometry to the given
-        order.  This method does not preserve the geometry footprint.
+        """Upgrade or downgrade the resolution to the given order.
+
+        This method does not preserve the geometry footprint.
 
         Returns
         -------
@@ -1016,17 +1002,11 @@ class HpxGeom(MapGeom):
 
         axes = copy.deepcopy(self.axes)
         return self.__class__(
-            2 ** order,
-            self.nest,
-            coordsys=self.coordsys,
-            region=self.region,
-            axes=axes,
-            conv=self.conv,
+            2 ** order, self.nest, coordsys=self.coordsys, region=self.region, axes=axes
         )
 
     def to_swapped(self):
-        """Make a copy of this geometry with a swapped ORDERING.  (NEST->RING
-        or vice versa)
+        """Geometry copy with swapped ORDERING (NEST->RING or vice versa).
 
         Returns
         -------
@@ -1040,16 +1020,11 @@ class HpxGeom(MapGeom):
             coordsys=self.coordsys,
             region=self.region,
             axes=axes,
-            conv=self.conv,
         )
 
     def to_image(self):
         return self.__class__(
-            np.max(self.nside),
-            self.nest,
-            coordsys=self.coordsys,
-            region=self.region,
-            conv=self.conv,
+            np.max(self.nside), self.nest, coordsys=self.coordsys, region=self.region
         )
 
     def to_cube(self, axes):
@@ -1059,7 +1034,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=self.region,
-            conv=self.conv,
             axes=axes,
         )
 
@@ -1102,7 +1076,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=idx,
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1131,7 +1104,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=idx,
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1145,7 +1117,6 @@ class HpxGeom(MapGeom):
                 self.nest,
                 coordsys=self.coordsys,
                 region=self.region,
-                conv=self.conv,
                 axes=copy.deepcopy(self.axes),
             )
 
@@ -1162,7 +1133,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=tuple(idx),
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1176,7 +1146,6 @@ class HpxGeom(MapGeom):
                 self.nest,
                 coordsys=self.coordsys,
                 region=self.region,
-                conv=self.conv,
                 axes=copy.deepcopy(self.axes),
             )
 
@@ -1189,7 +1158,6 @@ class HpxGeom(MapGeom):
             self.nest,
             coordsys=self.coordsys,
             region=tuple(idx),
-            conv=self.conv,
             axes=copy.deepcopy(self.axes),
         )
 
@@ -1202,7 +1170,6 @@ class HpxGeom(MapGeom):
         coordsys="CEL",
         region=None,
         axes=None,
-        conv="gadf",
         skydir=None,
         width=None,
     ):
@@ -1233,8 +1200,6 @@ class HpxGeom(MapGeom):
             ``skydir``.
         axes : list
             List of axes for non-spatial dimensions.
-        conv : str
-            Convention for FITS file format.
 
         Returns
         -------
@@ -1263,14 +1228,12 @@ class HpxGeom(MapGeom):
         elif isinstance(skydir, SkyCoord):
             lon, lat, frame = skycoord_to_lonlat(skydir, coordsys=coordsys)
         else:
-            raise ValueError("Invalid type for skydir: {!r}".format(type(skydir)))
+            raise ValueError(f"Invalid type for skydir: {type(skydir)!r}")
 
         if region is None and width is not None:
-            region = "DISK({:f},{:f},{:f})".format(lon, lat, width / 2.0)
+            region = f"DISK({lon},{lat},{width/2})"
 
-        return cls(
-            nside, nest=nest, coordsys=coordsys, region=region, axes=axes, conv=conv
-        )
+        return cls(nside, nest=nest, coordsys=coordsys, region=region, axes=axes)
 
     @staticmethod
     def identify_hpx_convention(header):
@@ -1339,7 +1302,7 @@ class HpxGeom(MapGeom):
 
         if header["PIXTYPE"] != "HEALPIX":
             raise ValueError(
-                "Header PIXTYPE must be 'HEALPIX'. Got: {}".format(header["PIXTYPE"])
+                f"Invalid header PIXTYPE: {header['PIXTYPE']} (must be HEALPIX)"
             )
 
         if header["ORDERING"] == "RING":
@@ -1348,9 +1311,7 @@ class HpxGeom(MapGeom):
             nest = True
         else:
             raise ValueError(
-                "Header ORDERING must be RING or NESTED. Got: {}".format(
-                    header["ORDERING"]
-                )
+                f"Invalid header ORDERING: {header['ORDERING']} (must be RING or NESTED)"
             )
 
         if hdu_bands is not None and "NSIDE" in hdu_bands.columns.names:
@@ -1375,9 +1336,7 @@ class HpxGeom(MapGeom):
             except KeyError:
                 region = None
 
-        return cls(
-            nside, nest, coordsys=coordsys, region=region, axes=axes, conv=convname
-        )
+        return cls(nside, nest, coordsys=coordsys, region=region, axes=axes)
 
     @classmethod
     def from_hdu(cls, hdu, hdu_bands=None):
@@ -1406,10 +1365,10 @@ class HpxGeom(MapGeom):
 
         return cls.from_header(hdu.header, hdu_bands=hdu_bands, pix=pix)
 
-    def make_header(self, **kwargs):
+    def make_header(self, conv="gadf", **kwargs):
         """Build and return FITS header for this HEALPIX map."""
         header = fits.Header()
-        conv = kwargs.get("conv", HPX_FITS_CONVENTIONS[self.conv])
+        conv = kwargs.get("conv", HPX_FITS_CONVENTIONS[conv])
 
         # FIXME: For some sparse maps we may want to allow EXPLICIT
         # with an empty region string
@@ -1496,10 +1455,10 @@ class HpxGeom(MapGeom):
             elif tokens[1] == "RING":
                 ipix_ring = int(tokens[3])
             else:
-                raise ValueError("Invalid ordering scheme: {!r}".format(tokens[1]))
+                raise ValueError(f"Invalid ordering scheme: {tokens[1]!r}")
             ilist = match_hpx_pix(nside, nest, nside_pix, ipix_ring)
         else:
-            raise ValueError("Invalid region type: {!r}".format(reg_type))
+            raise ValueError(f"Invalid region type: {reg_type!r}")
 
         return ilist
 
@@ -1593,7 +1552,7 @@ class HpxGeom(MapGeom):
 
     def get_idx(self, idx=None, local=False, flat=False):
         if idx is not None and np.any(np.array(idx) >= np.array(self.shape_axes)):
-            raise ValueError("Image index out of range: {!r}".format(idx))
+            raise ValueError(f"Image index out of range: {idx!r}")
 
         # Regular all- and partial-sky maps
         if self.is_regular:
@@ -1685,7 +1644,7 @@ class HpxGeom(MapGeom):
     def get_coord(self, idx=None, flat=False):
         pix = self.get_idx(idx=idx, flat=flat)
         coords = self.pix_to_coord(pix)
-        cdict = OrderedDict([("lon", coords[0]), ("lat", coords[1])])
+        cdict = {"lon": coords[0], "lat": coords[1]}
 
         for i, axis in enumerate(self.axes):
             cdict[axis.name] = coords[i + 2]
@@ -1713,22 +1672,22 @@ class HpxGeom(MapGeom):
         return Quantity(hp.nside2pixarea(self.nside), "sr")
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
         axes = ["skycoord"] + [_.name for _ in self.axes]
-        str_ += "\taxes       : {}\n".format(", ".join(axes))
-        str_ += "\tshape      : {}\n".format(self.data_shape[::-1])
-        str_ += "\tndim       : {}\n".format(self.ndim)
-        str_ += "\tnside      : {nside[0]}\n".format(nside=self.nside)
-        str_ += "\tnested     : {}\n".format(self.nest)
-        str_ += "\tcoordsys   : {}\n".format(self.coordsys)
-        str_ += "\tprojection : {}\n".format(self.projection)
         lon, lat = self.center_skydir.data.lon.deg, self.center_skydir.data.lat.deg
-        str_ += "\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n".format(lon=lon, lat=lat)
-        return str_
+
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\taxes       : {axes}\n"
+            f"\tshape      : {self.data_shape[::-1]}\n"
+            f"\tndim       : {self.ndim}\n"
+            f"\tnside      : {self.nside[0]}\n"
+            f"\tnested     : {self.nest}\n"
+            f"\tcoordsys   : {self.coordsys}\n"
+            f"\tprojection : {self.projection}\n"
+            f"\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n"
+        )
 
     def __eq__(self, other):
-        """Test equality between two `~gammapy.maps.HpxGeom`"""
         if not isinstance(other, self.__class__):
             return NotImplemented
 
@@ -1778,47 +1737,42 @@ class HpxToWcsMapping:
 
     @property
     def hpx(self):
-        """The HEALPIX projection"""
+        """HEALPIX projection."""
         return self._hpx
 
     @property
     def wcs(self):
-        """The WCS projection"""
+        """WCS projection."""
         return self._wcs
 
     @property
     def ipix(self):
-        """An array(nx,ny) of the global HEALPIX pixel indices for each WCS pixel"""
+        """An array(nx,ny) of the global HEALPIX pixel indices for each WCS pixel."""
         return self._ipix
 
     @property
     def mult_val(self):
-        """An array(nx,ny) of 1/number of WCS pixels pointing at each HEALPIX pixel"""
+        """An array(nx,ny) of 1/number of WCS pixels pointing at each HEALPIX pixel."""
         return self._mult_val
 
     @property
     def npix(self):
-        """A tuple(nx,ny) of the shape of the WCS grid"""
+        """A tuple(nx,ny) of the shape of the WCS grid."""
         return self._npix
 
     @property
     def lmap(self):
-        """An array(nx,ny) giving the mapping of the local HEALPIX pixel
-        indices for each WCS pixel.
-        """
+        """Array ``(nx, ny)`` mapping local HEALPIX pixel indices for each WCS pixel."""
         return self._lmap
 
     @property
     def valid(self):
-        """Array(nx,ny) of bools giving if each WCS pixel in inside the
-        HEALPIX region.
-        """
+        """Array ``(nx, ny)`` of bool: which WCS pixel in inside the HEALPIX region."""
         return self._valid
 
     @classmethod
     def create(cls, hpx, wcs):
-        """Create an object that maps pixels from HEALPix geometry ``hpx`` to
-        WCS geometry ``wcs``.
+        """Create HEALPix to WCS geometry pixel mapping.
 
         Parameters
         ----------
