@@ -1,18 +1,24 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import copy
-from collections import OrderedDict
+from functools import lru_cache
 import numpy as np
-from astropy.wcs import WCS
+import astropy.units as u
+from astropy.coordinates import Angle, SkyCoord
 from astropy.io import fits
 from astropy.nddata import Cutout2D
-from astropy.coordinates import SkyCoord, Angle
-from astropy.coordinates.angle_utilities import angular_separation
+from astropy.wcs import WCS
 from astropy.wcs.utils import proj_plane_pixel_scales
-import astropy.units as u
 from regions import SkyRegion
-from ..utils.wcs import get_resampled_wcs
-from .geom import MapGeom, MapCoord, pix_tuple_to_idx, skycoord_to_lonlat
-from .geom import get_shape, make_axes, find_and_read_bands, axes_pix_to_coord
+from .geom import (
+    Geom,
+    MapCoord,
+    axes_pix_to_coord,
+    find_and_read_bands,
+    get_shape,
+    make_axes,
+    pix_tuple_to_idx,
+    skycoord_to_lonlat,
+)
 from .utils import INVALID_INDEX
 
 __all__ = ["WcsGeom"]
@@ -58,6 +64,20 @@ def cast_to_shape(param, shape, dtype):
     return tuple(param)
 
 
+def get_resampled_wcs(wcs, factor, downsampled):
+    """
+    Get resampled WCS object.
+    """
+    wcs = wcs.deepcopy()
+
+    if not downsampled:
+        factor = 1.0 / factor
+
+    wcs.wcs.cdelt *= factor
+    wcs.wcs.crpix = (wcs.wcs.crpix - 0.5) / factor + 0.5
+    return wcs
+
+
 # TODO: remove this function, move code to the one caller below
 def _make_image_header(
     nxpix=100,
@@ -115,7 +135,7 @@ def _make_image_header(
     elif coordsys == "GAL":
         ctype1, ctype2 = "GLON-", "GLAT-"
     else:
-        raise ValueError("Unsupported coordsys: {!r}".format(coordsys))
+        raise ValueError(f"Unsupported coordsys: {coordsys!r}")
 
     pars = {
         "NAXIS": 2,
@@ -139,7 +159,7 @@ def _make_image_header(
     return header
 
 
-class WcsGeom(MapGeom):
+class WcsGeom(Geom):
     """Geometry class for WCS maps.
 
     This class encapsulates both the WCS transformation object and the
@@ -159,21 +179,17 @@ class WcsGeom(MapGeom):
         Reference pixel coordinate in each image plane.
     axes : list
         Axes for non-spatial dimensions
-    conv : {'gadf', 'fgst-ccube', 'fgst-template'}
-        Serialization format convention.  This sets the default format
-        that will be used when writing this geometry to a file.
     """
 
     _slice_spatial_axes = slice(0, 2)
     _slice_non_spatial_axes = slice(2, None)
     is_hpx = False
 
-    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None, conv="gadf"):
+    def __init__(self, wcs, npix, cdelt=None, crpix=None, axes=None):
         self._wcs = wcs
         self._coordsys = get_coordys(wcs)
         self._projection = get_projection(wcs)
-        self._conv = conv
-        self._axes = make_axes(axes, conv)
+        self._axes = make_axes(axes)
 
         if cdelt is None:
             cdelt = tuple(np.abs(self.wcs.wcs.cdelt))
@@ -267,11 +283,6 @@ class WcsGeom(MapGeom):
         """Tuple with image dimension in pixels in longitude and latitude."""
         return self._npix
 
-    @property
-    def conv(self):
-        """Name of default FITS convention associated with this geometry."""
-        return self._conv
-
     @property
     def axes(self):
         """List of non-spatial axes."""
@@ -336,7 +347,6 @@ class WcsGeom(MapGeom):
         axes=None,
         skydir=None,
         width=None,
-        conv="gadf",
     ):
         """Create a WCS geometry object.
 
@@ -378,9 +388,6 @@ class WcsGeom(MapGeom):
         refpix : tuple
             Reference pixel of the projection.  If None this will be
             set to the center of the map.
-        conv : string, optional
-            FITS format convention ('fgst-ccube', 'fgst-template',
-            'gadf').  Default is 'gadf'.
 
         Returns
         -------
@@ -404,7 +411,7 @@ class WcsGeom(MapGeom):
         elif isinstance(skydir, SkyCoord):
             xref, yref, frame = skycoord_to_lonlat(skydir, coordsys=coordsys)
         else:
-            raise ValueError("Invalid type for skydir: {!r}".format(type(skydir)))
+            raise ValueError(f"Invalid type for skydir: {type(skydir)!r}")
 
         if width is not None:
             width = _check_width(width)
@@ -440,7 +447,7 @@ class WcsGeom(MapGeom):
             yrefpix=refpix[1],
         )
         wcs = WCS(header)
-        return cls(wcs, npix, cdelt=binsz, axes=axes, conv=conv)
+        return cls(wcs, npix, cdelt=binsz, axes=axes)
 
     @classmethod
     def from_header(cls, header, hdu_bands=None):
@@ -463,16 +470,8 @@ class WcsGeom(MapGeom):
         for i in range(naxis - 2):
             wcs = wcs.dropaxis(2)
 
-        axes = find_and_read_bands(hdu_bands, header)
+        axes = find_and_read_bands(hdu_bands)
         shape = tuple([ax.nbin for ax in axes])
-        conv = "gadf"
-
-        # Discover FITS convention
-        if hdu_bands is not None:
-            if hdu_bands.name == "EBOUNDS":
-                conv = "fgst-ccube"
-            elif hdu_bands.name == "ENERGIES":
-                conv = "fgst-template"
 
         if hdu_bands is not None and "NPIX" in hdu_bands.columns.names:
             npix = hdu_bands.data.field("NPIX").reshape(shape + (2,))
@@ -487,7 +486,7 @@ class WcsGeom(MapGeom):
             npix = (header["NAXIS1"], header["NAXIS2"])
             cdelt = None
 
-        return cls(wcs, npix, cdelt=cdelt, axes=axes, conv=conv)
+        return cls(wcs, npix, cdelt=cdelt, axes=axes)
 
     def _make_bands_cols(self, hdu=None, conv=None):
 
@@ -528,8 +527,8 @@ class WcsGeom(MapGeom):
         self._fill_header_from_axes(header)
         shape = "{},{}".format(np.max(self.npix[0]), np.max(self.npix[1]))
         for ax in self.axes:
-            shape += ",{}".format(ax.nbin)
-        header["WCSSHAPE"] = "({})".format(shape)
+            shape += f",{ax.nbin}"
+        header["WCSSHAPE"] = f"({shape})"
         return header
 
     def get_image_shape(self, idx):
@@ -539,6 +538,7 @@ class WcsGeom(MapGeom):
         else:
             return int(self.npix[0][idx]), int(self.npix[1][idx])
 
+    @lru_cache()
     def get_idx(self, idx=None, flat=False):
         pix = self.get_pix(idx=idx, mode="center")
         if flat:
@@ -565,6 +565,7 @@ class WcsGeom(MapGeom):
         pix = np.meshgrid(*pix[::-1], indexing="ij")[::-1]
         return pix
 
+    @lru_cache()
     def get_pix(self, idx=None, mode="center"):
         """Get map pix coordinates from the geometry.
 
@@ -585,7 +586,8 @@ class WcsGeom(MapGeom):
             _[~m] = INVALID_INDEX.float
         return pix
 
-    def get_coord(self, idx=None, flat=False, mode="center"):
+    @lru_cache()
+    def get_coord(self, idx=None, flat=False, mode="center", coordsys=None):
         """Get map coordinates from the geometry.
 
         Parameters
@@ -606,9 +608,12 @@ class WcsGeom(MapGeom):
             coords = tuple([c[is_finite] for c in coords])
 
         axes_names = ["lon", "lat"] + [ax.name for ax in self.axes]
-        cdict = OrderedDict(zip(axes_names, coords))
+        cdict = dict(zip(axes_names, coords))
 
-        return MapCoord.create(cdict, coordsys=self.coordsys)
+        if coordsys is None:
+            coordsys = self.coordsys
+
+        return MapCoord.create(cdict, coordsys=self.coordsys).to_coordsys(coordsys)
 
     def coord_to_pix(self, coords):
         coords = MapCoord.create(coords, coordsys=self.coordsys)
@@ -647,6 +652,11 @@ class WcsGeom(MapGeom):
         else:
             coords = self._wcs.wcs_pix2world(pix[0], pix[1], 0)
 
+        coords = [
+            u.Quantity(coords[0], unit="deg", copy=False),
+            u.Quantity(coords[1], unit="deg", copy=False),
+        ]
+
         coords += axes_pix_to_coord(self.axes, pix[self._slice_non_spatial_axes])
         return tuple(coords)
 
@@ -681,6 +691,7 @@ class WcsGeom(MapGeom):
         idx = self.coord_to_idx(coords)
         return np.all(np.stack([t != INVALID_INDEX.int for t in idx]), axis=0)
 
+    @lru_cache()
     def to_image(self):
         npix = (np.max(self._npix[0]), np.max(self._npix[1]))
         cdelt = (np.max(self._cdelt[0]), np.max(self._cdelt[1]))
@@ -712,27 +723,68 @@ class WcsGeom(MapGeom):
         cdelt = copy.deepcopy(self._cdelt)
         return self.__class__(wcs, npix, cdelt=cdelt, axes=copy.deepcopy(self.axes))
 
-    def downsample(self, factor):
+    def downsample(self, factor, axis=None):
+        if axis is None:
+            if np.any(np.mod(self.npix, factor) > 0):
+                raise ValueError(
+                    f"Spatial shape not divisible by factor {factor!r} in all axes."
+                    f" You need to pad prior to calling downsample."
+                )
 
-        if not np.all(np.mod(self.npix[0], factor) == 0) or not np.all(
-            np.mod(self.npix[1], factor) == 0
-        ):
-            raise ValueError(
-                "Data shape not divisible by factor {!r} in all axes."
-                " You need to pad prior to calling downsample.".format(factor)
-            )
+            npix = (self.npix[0] / factor, self.npix[1] / factor)
+            cdelt = (self._cdelt[0] * factor, self._cdelt[1] * factor)
+            wcs = get_resampled_wcs(self.wcs, factor, True)
+            return self._init_copy(wcs=wcs, npix=npix, cdelt=cdelt)
+        else:
+            if not self.is_regular:
+                raise NotImplementedError(
+                    "Upsampling in non-spatial axes not supported for irregular geometries"
+                )
 
-        npix = (self.npix[0] / factor, self.npix[1] / factor)
-        cdelt = (self._cdelt[0] * factor, self._cdelt[1] * factor)
-        wcs = get_resampled_wcs(self.wcs, factor, True)
-        return self.__class__(wcs, npix, cdelt=cdelt, axes=copy.deepcopy(self.axes))
+            axes = copy.deepcopy(self.axes)
+            idx = self.get_axis_index_by_name(axis)
+            axes[idx] = axes[idx].downsample(factor)
+            return self._init_copy(axes=axes)
 
-    def upsample(self, factor):
-        npix = (self.npix[0] * factor, self.npix[1] * factor)
-        cdelt = (self._cdelt[0] / factor, self._cdelt[1] / factor)
-        wcs = get_resampled_wcs(self.wcs, factor, False)
-        return self.__class__(wcs, npix, cdelt=cdelt, axes=copy.deepcopy(self.axes))
+    def upsample(self, factor, axis=None):
+        if axis is None:
+            npix = (self.npix[0] * factor, self.npix[1] * factor)
+            cdelt = (self._cdelt[0] / factor, self._cdelt[1] / factor)
+            wcs = get_resampled_wcs(self.wcs, factor, False)
+            return self._init_copy(wcs=wcs, npix=npix, cdelt=cdelt)
+        else:
+            if not self.is_regular:
+                raise NotImplementedError(
+                    "Upsampling in non-spatial axes not supported for irregular geometries"
+                )
+            axes = copy.deepcopy(self.axes)
+            idx = self.get_axis_index_by_name(axis)
+            axes[idx] = axes[idx].upsample(factor)
+            return self._init_copy(axes=axes)
 
+    def to_binsz(self, binsz):
+        """Change pixel size of the geometry
+
+        Parameters
+        ----------
+        binsz : float or tuple or list
+            New pixel size in degree.
+
+        Returns
+        -------
+        geom : `WcsGeom`
+            Geometry with new pixel size.
+        """
+        kwargs = {}
+        kwargs["skydir"] = self.center_skydir
+        kwargs["binsz"] = binsz
+        kwargs["width"] = self.width
+        kwargs["proj"] = self.projection
+        kwargs["coordsys"] = self.coordsys
+        kwargs["axes"] = copy.deepcopy(self.axes)
+        return self.create(**kwargs)
+
+    @lru_cache()
     def solid_angle(self):
         """Solid angle array (`~astropy.units.Quantity` in ``sr``).
 
@@ -742,29 +794,45 @@ class WcsGeom(MapGeom):
 
             WcsGeom.to_image().solid_angle()
         """
-        coord = self.get_coord(mode="edges")
-        lon = coord.lon * np.pi / 180.0
-        lat = coord.lat * np.pi / 180.0
+        coord = self.get_coord(mode="edges").skycoord
 
-        # Compute solid angle across centres of the pixels, approximating it
-        # as a rectangle
-        # First index is "y", second index is "x"
-        # TODO: Calculate actual solid angle between two great circles? Here are two references
-        # suggesting more precise methods:
-        # https://mail.python.org/pipermail/astropy/2013-December/002632.html
-        # https://cta-redmine.irap.omp.eu/issues/1017
-        lon_centres = (lon[..., :-1, :-1] + lon[..., 1:, 1:]) / 2
-        lat_centres = (lat[..., :-1, :-1] + lat[..., 1:, 1:]) / 2
+        # define pixel corners
+        low_left = coord[..., :-1, :-1]
+        low_right = coord[..., 1:, :-1]
+        up_left = coord[..., :-1, 1:]
+        up_right = coord[..., 1:, 1:]
 
-        ymid_xlo = lon[..., :-1, :-1], lat_centres
-        ymid_xhi = lon[..., :-1, 1:], lat_centres
-        ylo_xmid = lon_centres, lat[..., :-1, 1:]
-        yhi_xmid = lon_centres, lat[..., 1:, :-1]
+        # compute side lengths
+        low = low_left.separation(low_right)
+        left = low_left.separation(up_left)
+        up = up_left.separation(up_right)
+        right = low_right.separation(up_right)
 
-        dx = angular_separation(*(ymid_xlo + ymid_xhi))
-        dy = angular_separation(*(ylo_xmid + yhi_xmid))
+        # compute enclosed angles
+        angle_low_right = low_right.position_angle(up_right) - low_right.position_angle(
+            low_left
+        )
+        angle_up_left = up_left.position_angle(up_right) - low_left.position_angle(
+            up_left
+        )
 
-        return u.Quantity(dx * dy, "sr", copy=False)
+        # compute area assuming a planar triangle
+        area_low_right = 0.5 * low * right * np.sin(angle_low_right)
+        area_up_left = 0.5 * up * left * np.sin(angle_up_left)
+
+        return u.Quantity(area_low_right + area_up_left, "sr", copy=False)
+
+    @lru_cache()
+    def bin_volume(self):
+        """Bin volume (`~astropy.units.Quantity`)"""
+        bin_volume = self.to_image().solid_angle()
+
+        for idx, ax in enumerate(self.axes):
+            shape = self.ndim * [1]
+            shape[-(idx + 3)] = -1
+            bin_volume = bin_volume * ax.bin_width.reshape(tuple(shape))
+
+        return bin_volume
 
     def separation(self, center):
         """Compute sky separation wrt a given center.
@@ -878,21 +946,20 @@ class WcsGeom(MapGeom):
         return mask
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
         axes = ["lon", "lat"] + [_.name for _ in self.axes]
-        str_ += "\taxes       : {}\n".format(", ".join(axes))
-        str_ += "\tshape      : {}\n".format(self.data_shape[::-1])
-        str_ += "\tndim       : {}\n".format(self.ndim)
-        str_ += "\tcoordsys   : {}\n".format(self.coordsys)
-        str_ += "\tprojection : {}\n".format(self.projection)
         lon = self.center_skydir.data.lon.deg
         lat = self.center_skydir.data.lat.deg
-        str_ += "\tcenter     : {:.1f} deg, {:.1f} deg\n".format(lon, lat)
-        str_ += "\twidth      : {width[0][0]:.1f} x {width[1][0]:.1f} " "deg\n".format(
-            width=self.width
+
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\taxes       : {axes}\n"
+            f"\tshape      : {self.data_shape[::-1]}\n"
+            f"\tndim       : {self.ndim}\n"
+            f"\tcoordsys   : {self.coordsys}\n"
+            f"\tprojection : {self.projection}\n"
+            f"\tcenter     : {lon:.1f} deg, {lat:.1f} deg\n"
+            f"\twidth      : {self.width[0][0]:.1f} x {self.width[1][0]:.1f}\n"
         )
-        return str_
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -912,14 +979,17 @@ class WcsGeom(MapGeom):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return id(self)
+
 
 def pix2world(wcs, cdelt, crpix, pix):
-    """Perform pixel to world coordinate transformation for a WCS
-    projection with a given pixel size (CDELT) and reference pixel
-    (CRPIX).  This method can be used to perform WCS transformations
+    """Perform pixel to world coordinate transformation.
+
+    For a WCS projection with a given pixel size (CDELT) and reference pixel
+    (CRPIX). This method can be used to perform WCS transformations
     for projections with different pixelizations but the same
-    reference coordinate (CRVAL), projection type, and coordinate
-    system.
+    reference coordinate (CRVAL), projection type, and coordinate system.
 
     Parameters
     ----------
