@@ -2,18 +2,18 @@
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
-from astropy.table import Table
 import astropy.units as u
-from ...catalog.fermi import SourceCatalog3FGL
-from ...utils.testing import (
-    requires_dependency,
-    requires_data,
+from astropy.table import Table
+from gammapy.catalog.fermi import SourceCatalog3FGL
+from gammapy.modeling import Fit
+from gammapy.modeling.models import PowerLawSpectralModel, SpectralModel
+from gammapy.spectrum import FluxPoints, FluxPointsDataset
+from gammapy.utils.testing import (
     assert_quantity_allclose,
     mpl_plot_check,
+    requires_data,
+    requires_dependency,
 )
-from ...utils.fitting import Fit
-from ..models import PowerLaw, SpectralModel
-from ..flux_point import FluxPoints, FluxPointsDataset
 
 FLUX_POINTS_FILES = [
     "diff_flux_points.ecsv",
@@ -231,14 +231,25 @@ def test_compute_flux_points_dnde_fermi():
 
 
 @pytest.fixture()
-def fit():
+def fit(dataset):
+    return Fit(dataset)
+
+
+@pytest.fixture()
+def dataset():
     path = "$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits"
     data = FluxPoints.read(path)
     data.table["e_ref"] = data.e_ref.to("TeV")
-
-    model = PowerLaw(index=2.3, amplitude="2e-13 cm-2 s-1 TeV-1", reference="1 TeV")
+    model = PowerLawSpectralModel(
+        index=2.3, amplitude="2e-13 cm-2 s-1 TeV-1", reference="1 TeV"
+    )
     dataset = FluxPointsDataset(model, data)
-    return Fit(dataset)
+    return dataset
+
+
+@requires_data()
+def test_flux_point_dataset_str(dataset):
+    assert "FluxPointsDataset" in str(dataset)
 
 
 @requires_data()
@@ -295,4 +306,4 @@ class TestFluxPointFit:
         fp_dataset = fit.datasets.datasets[0]
 
         with mpl_plot_check():
-            fp_dataset.peek()
+            fp_dataset.peek(method="diff/model")
