@@ -13,57 +13,42 @@ import gammapy.astro.population
 print("import: 'gammapy.astro.source'")
 import gammapy.astro.source
 
-print("import: 'gammapy.background'")
-import gammapy.background
-
 print("import: 'gammapy.catalog'")
 import gammapy.catalog
 
 print("import: 'gammapy.cube'")
 import gammapy.cube
 
-print("import: 'gammapy.cube.models'")
-import gammapy.cube.models
-
 print("import: 'gammapy.data'")
 import gammapy.data
 
 print("import: 'gammapy.detect'")
 import gammapy.detect
 
-print("import: 'gammapy.extern'")
-import gammapy.extern
-
-print("import: 'gammapy.image'")
-import gammapy.image
-
-print("import: 'gammapy.image.models'")
-import gammapy.image.models
-
 print("import: 'gammapy.irf'")
 import gammapy.irf
 
 print("import: 'gammapy.maps'")
 import gammapy.maps
 
+print("import: 'gammapy.modeling'")
+import gammapy.modeling
+
+print("import: 'gammapy.modeling.models'")
+import gammapy.modeling.models
+
 print("import: 'gammapy.scripts'")
 import gammapy.scripts
 
 print("import: 'gammapy.spectrum'")
 import gammapy.spectrum
 
-print("import: 'gammapy.spectrum.models'")
-import gammapy.spectrum.models
-
 print("import: 'gammapy.stats'")
 import gammapy.stats
 
 print("import: 'gammapy.time'")
 import gammapy.time
 
-print("import: 'gammapy.time.models'")
-import gammapy.time.models
-
 print("import: 'gammapy.utils'")
 import gammapy.utils
 
@@ -73,6 +58,3 @@ import gammapy.utils.coordinates
 print("import: 'gammapy.utils.random'")
 import gammapy.utils.random
 
-print("import: 'gammapy.utils.fitting'")
-import gammapy.utils.fitting
-
